@@ -9,11 +9,13 @@ use vars qw($VERSION);
 
 use Carp;
 
-$VERSION = '0.11_03';
+BEGIN { 
+	$VERSION = '0.17_03' 
+	}; # needed right away to set defaults at compile-time
 
 =head1 NAME
 
-CPAN::PackageDetails - Create or read 02.packages.details.txt.gz
+CPAN::PackageDetails - Create or read 02packages.details.txt.gz
 
 =head1 SYNOPSIS
 
@@ -22,7 +24,6 @@ CPAN::PackageDetails - Create or read 02.packages.details.txt.gz
 	# read an existing file #####################
 	my $package_details = CPAN::PackageDetails->read( $filename );
 	
-	my $creator    = $package_details->creator;  # See CPAN::PackageDetails::Header too
 	my $count      = $package_details->count;
 	
 	my $records    = $package_details->entries;
@@ -40,12 +41,12 @@ CPAN::PackageDetails - Create or read 02.packages.details.txt.gz
 		package => qr/^Test::/, # or a string
 		author  => 'OVID',      # case insenstive
 		path    =>  qr/foo/,
-		)
+		);
 	
 	# create a new file #####################
 	my $package_details = CPAN::PackageDetails->new( 
-		file         => "02.packages.details.txt",
-		url          => "http://example.com/MyCPAN/modules/02.packages.details.txt",
+		file         => "02packages.details.txt",
+		url          => "http://example.com/MyCPAN/modules/02packages.details.txt",
 		description  => "Package names for my private CPAN",
 		columns      => "package name, version, path",
 		intended_for => "My private CPAN",
@@ -68,11 +69,11 @@ CPAN::PackageDetails - Create or read 02.packages.details.txt.gz
 	
 =head1 DESCRIPTION
 
-CPAN uses an index file, 02.packages.details.txt.gz, to map package names to
+CPAN uses an index file, 02packages.details.txt.gz, to map package names to
 distribution files. Using this module, you can get a data structure of that
 file, or create your own.
 
-There are two parts to the 02.packages.details.txt.gz: a header and the index.
+There are two parts to the 02packages.details.txt.gz: a header and the index.
 This module uses a top-level C<CPAN::PackageDetails> object to control
 everything and comprise an C<CPAN::PackageDetails::Header> and
 C<CPAN::PackageDetails::Entries> object. The C<CPAN::PackageDetails::Entries>
@@ -93,7 +94,7 @@ Entry objects.
 
 =item new
 
-Create a new 02.packages.details.txt.gz file. The C<default_headers>
+Create a new 02packages.details.txt.gz file. The C<default_headers>
 method shows you which values you can pass to C<new>. For instance:
 
 	my $package_details = CPAN::PackageDetails->new(
@@ -123,7 +124,7 @@ Sets up the object. C<new> calls this automatically for you.
 Returns the hash of header fields and their default values:
 
 	file            "02packages.details.txt"
-	url             "http://example.com/MyCPAN/modules/02.packages.details.txt"
+	url             "http://example.com/MyCPAN/modules/02packages.details.txt"
 	description     "Package names for my private CPAN"
 	columns         "package name, version, path"
 	intended_for    "My private CPAN"
@@ -141,12 +142,23 @@ Write the date in PAUSE format. For example:
 	
 =cut
 
-sub format_date { join ", ", split /\s+/, scalar gmtime, 2 }
+sub format_date 
+	{ 
+	my( $second, $minute, $hour, $date, $monnum, $year, $wday )  = gmtime;
+	$year += 1900;
+
+	my $day   = ( qw(Sun Mon Tue Wed Thu Fri Sat) )[$wday];
+	my $month = ( qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec) )[$monnum];
+
+	sprintf "%s, %02d %s %4d %02d:%02d:%02d GMT",
+		$day, $date, $month, $year, $hour, $minute, $second;
+	}
+
 
 BEGIN {
 my %defaults = (
 	file            => "02packages.details.txt",
-	url             => "http://example.com/MyCPAN/modules/02.packages.details.txt",
+	url             => "http://example.com/MyCPAN/modules/02packages.details.txt",
 	description     => "Package names for my private CPAN",
 	columns         => "package name, version, path",
 	intended_for    => "My private CPAN",
@@ -196,7 +208,7 @@ sub CPAN::PackageDetails::Header::AUTOLOAD
 # them to the right delegate
 my %Dispatch = (
 		header  => { map { $_, 1 } qw(get_header set_header header_exists columns_as_list) },
-		entries => { map { $_, 1 } qw(add_entry count) },
+		entries => { map { $_, 1 } qw(add_entry count as_unique_sorted_list) },
 	#	entry   => { map { $_, 1 } qw() },
 		);
 		
@@ -279,7 +291,7 @@ sub init
 
 =item read( FILE )
 
-Read an existing 02.packages.details.txt.gz file.
+Read an existing 02packages.details.txt.gz file.
 
 While parsing, it modifies the field names to map them to Perly
 identifiers. The field is lowercased, and then hyphens become
@@ -404,7 +416,7 @@ sub DESTROY {}
 
 =head2 Headers
 
-The 02.packages.details.txt.gz header is a short preamble that give information
+The 02packages.details.txt.gz header is a short preamble that give information
 about the creation of the file, its intended use, and the number of entries in
 the file. It looks something like:
 
@@ -471,6 +483,7 @@ so it can put it in the "Line-Count" header.
 	CPAN::PackageDetails::Header->new(
 		_entries => $entries_object,
 		)
+
 =cut
 
 sub new { 
@@ -606,7 +619,7 @@ sub as_string
 		push @lines, "$out_field: $value";
 		}
 		
-	push @lines, "Line-Count: " . $self->_entries->count;
+	push @lines, "Line-Count: " . $self->_entries->as_unique_sorted_list;
 	
 	join "\n", sort( @lines ), "\n";
 	}
@@ -717,7 +730,9 @@ sub columns { @{ $_[0]->{columns} } };
 
 =item count
 
-Returns the number of entries.
+Returns the number of entries. This is not the same as the number of
+lines that would show up in the F<02packages.details.txt> file since
+this method counts duplicates as well. 
 
 =cut
 
@@ -742,8 +757,31 @@ sub add_entry
 	{
 	my( $self, %args ) = @_;
 
+	$self->_mark_as_dirty;
+	
+	# The column name has a space in it, but that looks weird in a 
+	# hash constructor and I keep doing it wrong. If I type "package_name"
+	# I'll just make it work.
+	if( exists $args{package_name} )
+		{
+		$args{'package name'} = $args{package_name};
+		delete $args{package_name};
+		}
+		
+	unless( defined $args{'package name'} )
+		{
+		carp "No 'package name' parameter!";
+		return;
+		}
+		
+	# should check for allowed columns here
 	push @{ $self->{entries} }, $self->entry_class->new( %args );
 	}
+
+sub _mark_as_dirty
+	{
+	delete $_[0]->{sorted};
+	}
 	
 =item as_string
 
@@ -756,23 +794,56 @@ sub as_string
 	{
 	my( $self ) = @_;
 	
-	my $entries;
+	my $string;
 	
-	foreach my $entry ( @{ $self->entries } )
+	foreach my $entry ( $self->as_unique_sorted_list )
 		{
-		$entries .= $entry->as_string( $self->columns );
+		$string .= $entry->as_string( $self->columns );
 		}
 	
-	$entries || '';
+	$string || '';
+	}
+
+=item as_unique_sorted_list
+
+In list context, this returns a list of entries sorted by package name
+and version. Each package exists exactly once in the list and with the
+largest version number seen.
+
+In scalar context this returns the count of the number of unique entries.
+
+=cut
+
+sub as_unique_sorted_list
+	{
+	my( $self ) = @_;
+
+	unless( ref $self->{sorted} eq ref [] )
+		{
+		my %Seen;
+
+		my( $k1, $k2 ) = ( $self->columns )[0,1];
+
+		$self->{sorted} = [
+			grep { ! $Seen{ $_->{$k1} }++ }
+				sort { $a->{$k1} cmp $b->{$k1} || $b->{$k2} <=> $a->{$k2} } 
+				@{ $self->entries }
+			];
+		}
+		
+	return wantarray ? 
+		@{ $self->{sorted} } 
+			:
+		scalar  @{ $self->{sorted} }
 	}
 
 }
-
+	
 =back
 
 =head2 Entry
 
-An entry is a single line from 02.packages.details.txt that maps a
+An entry is a single line from F<02packages.details.txt> that maps a
 package name to a source. It's a whitespace-separated list that
 has the values for the column identified in the "columns" field
 in the header.
@@ -836,13 +907,19 @@ sub new
 Formats the Entry as text. It joins with whitespace the values for the 
 column names you pass it. You get the newline automatically.
 
+Any values that are not defined (or the empty string) turn into the
+literal string 'undef' to preserve the columns in the output.
+
 =cut
 
 sub as_string
 	{
 	my( $self, @columns ) = @_;
 	
-	return join( "\t", map { $self->{$_} } @columns ) . "\n";
+	# can't check defined() because that let's the empty string through
+	return join( "\t", 
+		map { length $self->{$_} ? $self->{$_} : 'undef' } @columns 
+		) . "\n";
 	}	
 
 }
@@ -867,7 +944,7 @@ brian d foy, C<< <bdfoy@cpan.org> >>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2008, brian d foy, All Rights Reserved.
+Copyright (c) 2009, brian d foy, All Rights Reserved.
 
 You may redistribute this under the same terms as Perl itself.
 
